package top.ninwoo.cloud.service.dbctrans;

import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import top.ninwoo.api.CnfData;
import top.ninwoo.api.Picture;
import top.ninwoo.cloud.service.TransService;
import top.ninwoo.cloud.service.bpso.Bpso;
import top.ninwoo.cloud.service.bpso.InterSlotDijstraV2;
import top.ninwoo.cloud.service.csv.Wteg;
import top.ninwoo.cloud.service.findService.FindServiceImpl;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.concurrent.ConcurrentHashMap;


@Service
public class DbcTrans implements TransService {
    @Resource
    private RestTemplate restTemplate;

    /**
     * 打包picture
     *
     * @return picture列表
     */
    @Override
    public Picture packagePic(BufferedImage bufferedImage, String fileId) {
        Picture picture = new Picture();
        //转化为byte[]数组
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            ImageIO.write(bufferedImage, "jpg", out);
        } catch (IOException e) {
            e.printStackTrace();
        }
        //设置图片数据
        picture.setData(out.toByteArray());
        //设置文件名
        picture.setPicId(fileId);
        return picture;
    }

    /**
     * 打包成数据包
     *
     * @param picture
     */
    @Override
    public LinkedList<CnfData> packageData(Picture picture) throws ParseException {
        return null;
    }

    /**
     * 打包成数据包
     */
    //satename 为需要完成分散计算的卫星节点
    @Override
    public LinkedList<CnfData> packageData(Picture picture,String satename) throws ParseException {
        LinkedList<CnfData> cnfDatas = new LinkedList<>();
        CnfData cnfData = new CnfData();
        FindServiceImpl findService = new FindServiceImpl();
        //判断npe
        if(picture == null){
            System.out.println("出错");
            return null;
        }
        //设备数
        int num = Bpso.num;
        //时隙大小
        double slot = Bpso.slot;
        //时空拓展图大小
        int dev = 3 * num;
        //1组装最短路矩阵
        double[][] edgeWeight = Wteg.getWteg(dev);

        InterSlotDijstraV2 dijs = new InterSlotDijstraV2(num, slot);
        InterSlotDijstraV2.pathRecord pathRecord = dijs.dijstra(edgeWeight, 0);
        //2加权最短路由矩阵
        double[][] lt = pathRecord.getPathWeight();
        //最短路由矩阵
        int[][] lastHop = pathRecord.getLastHop();

        //路径表
        String[] path = new String[1];
        path[0] = "D1->"+satename;
        cnfData.setPathTable(path);
        //任务表
        ConcurrentHashMap<String,String> taskTable = new ConcurrentHashMap();
        taskTable.put(satename,"distributeProcess");
        cnfData.setTaskTable(taskTable);
        //T3路由表 todo
        String[][] route = new String[lastHop.length][lastHop[0].length];
        for(int i =0;i<lastHop.length;i++){
            for (int j=0;j<lastHop[0].length;j++){
                route[i][j] = "sate"+(lastHop[i][j]);
                if (route[i][j].equals("sate0")){
                    route[i][j]="sate"+(j);
                }
                if (route[i][j].equals("sate0")){
                    route[i][j]="D1";
                }
                //要修改
                //todo
                if (route[i][j].equals("sate31")){
                    route[i][j]="D2";
                }
            }
        }
        //不太正确
        cnfData.setRoute(route);
        //图片
        cnfData.setPicture(picture);
        //ip池
        cnfData.setIpPool(findService.findIPool());
        String[] routeself = findService.findRoute(cnfData.getCurrentSataHop(), cnfData.getRoute());
        ArrayList<String> destinSataHops = findService.findDestinSataHops(path,"D1");
        for(String des : destinSataHops) {
            CnfData transferPackage = cnfData;
            //目的
            transferPackage.setDestinSataHop(des);
            //当前
            transferPackage.setNextSataHop(findService.findNextHop(des, routeself));
            //下一跳
            transferPackage.setCurrentSataHop(transferPackage.getNextSataHop());
            //ip
            transferPackage.setNextIPHop(findService.findIp(transferPackage.getNextSataHop(),transferPackage.getIpPool()));
            //终止节点
            transferPackage.setEndSateHop("sate11");
            //加权路径路由
            transferPackage.setLt(lt);
            //返回一个数据包
            cnfDatas.add(transferPackage);
        }
        return cnfDatas;
    }

    /**
     * 发送数据包
     *
     * @param cnfData
     * @param url
     */
    @Override
    public String sendData(CnfData cnfData, String url) {
        String IP = url + ":8083";
        ResponseEntity<String> res = restTemplate.postForEntity("http://" + IP + "/transfer", cnfData, String.class);
        if (!res.getStatusCode().is2xxSuccessful()) {
            throw new RuntimeException("send Error!");
        }
        return "sucess";
    }

    /**
     * 发送得到cpu计算能力的指令
     *
     * @param url
     */
    @Override
    public Double sendCpuCtr(String url) {
        System.out.println("kaishi fasong ");
        String IP = url + ":8083";
        Double res = restTemplate.getForObject("http://" + IP + "/getCpuComputing", Double.class);
        System.out.println(res);
        return res;
    }

    @Override
    public String[][] getIt() throws ParseException {
        //设备数
        int num = Bpso.num;
        //时隙大小
        double slot = Bpso.slot;
        //时空拓展图大小
        int dev = 3 * num;
        //1组装最短路矩阵
        double[][] edgeWeight = Wteg.getWteg(dev);

        InterSlotDijstraV2 dijs = new InterSlotDijstraV2(num, slot);
        InterSlotDijstraV2.pathRecord pathRecord = dijs.dijstra(edgeWeight, 0);
        //2最短路由矩阵
        double[][] lt = pathRecord.getPathWeight();
        int[][] lastHop = pathRecord.getLastHop();
        for (int j = 0;j<lastHop.length;j++){
            for(int i = 0;i<lastHop.length;i++){
                int tmp0 = lastHop[j][i];
                while (tmp0 != 0){
                    lastHop[j][i] = tmp0;
                    tmp0 = lastHop[j][tmp0];
                }
            }
        }
        //T3路由表
        String[][] routetest = new String[lastHop.length][lastHop[0].length];
        for (int l = 0; l < lastHop.length/num; l++) {
            for (int i = l * num; i < l*num+num; i++) {
                for (int j = l * num; j < lastHop.length; j++) {
                    routetest[i][j] = "sate" + (lastHop[i][j]);
                    //到自己
                    if((i != 0 || i != num*l)&(routetest[i][j].equals("sate0"))){
                        routetest[i][j] = "sate"+i;
                    }
                    //表示不可达
                    if(j==num*l){
                        routetest[i][j] = null;
                    }
                }
            }
        }
        return routetest;
    }
}
